@@ -1,3 +1,17 @@
+# Copyright (C) 2020-2022 Arnaud Belcour - Inria Dyliss
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>
+
 import csv
 import json
 import logging
@@ -20,10 +34,10 @@ logger = logging.getLogger(__name__)
 
 
 def rest_query_uniprot_to_retrieve_function(protein_queries, beta=None):
-    """REST query to get annotation from proteomes.
+    """REST query to get annotation from proteins.
 
     Args:
-        proteomes (list): list of proteomes
+        protein_queries (list): list of proteins
         beta (bool): option to use the new API of UniProt (in beta can be unstable)
 
     Returns:
@@ -905,11 +919,11 @@ def annotate_proteins(input_folder, output_folder, uniprot_sparql_endpoint, prop
     else:
         uniprot_releases = get_rest_uniprot_release(options)
 
-    proteome_cluster_tax_id_file = os.path.join(input_folder, 'proteome_cluster_tax_id.tsv')
+    proteome_tax_id_file = os.path.join(input_folder, 'proteome_tax_id.tsv')
 
     if uniprot_sparql_endpoint:
         input_proteomes = {}
-        with open(proteome_cluster_tax_id_file, 'r') as tax_id_file:
+        with open(proteome_tax_id_file, 'r') as tax_id_file:
             csvreader = csv.reader(tax_id_file, delimiter='\t')
             next(csvreader)
             for line in csvreader:
@@ -973,7 +987,7 @@ def annotate_proteins(input_folder, output_folder, uniprot_sparql_endpoint, prop
 
     # Create mpwt taxon ID file.
     clustering_taxon_id = {}
-    with open(proteome_cluster_tax_id_file, 'r') as input_taxon_id_file:
+    with open(proteome_tax_id_file, 'r') as input_taxon_id_file:
         taxon_id_csvreader = csv.reader(input_taxon_id_file, delimiter='\t')
         next(taxon_id_csvreader)
         for line in taxon_id_csvreader:
