@@ -1,3 +1,17 @@
+# Copyright (C) 2020-2022 Arnaud Belcour - Inria Dyliss
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>
+
 import csv
 import gzip
 import json
@@ -336,8 +350,8 @@ def make_clustering(proteome_folder, output_folder, nb_cpu, clust_threshold, mms
         if remove_tmp:
             shutil.rmtree(mmseqs_tmp_cluster)
 
-    proteome_taxon_id_file = os.path.join(proteome_folder, 'proteome_cluster_tax_id.tsv')
-    clustering_taxon_id_file = os.path.join(output_folder, 'proteome_cluster_tax_id.tsv')
+    proteome_taxon_id_file = os.path.join(proteome_folder, 'proteome_tax_id.tsv')
+    clustering_taxon_id_file = os.path.join(output_folder, 'proteome_tax_id.tsv')
 
     if os.path.exists(clustering_taxon_id_file):
         if not os.path.samefile(proteome_taxon_id_file, clustering_taxon_id_file):
