@@ -1,3 +1,17 @@
+# Copyright (C) 2020-2022 Arnaud Belcour - Inria Dyliss
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>
+
 import csv
 import json
 import logging
@@ -70,7 +84,7 @@ def perform_workflow(input_file, output_folder, busco_percentage_keep=80, ignore
                         limit_maximal_number_proteomes=99, rank_limit=None,
                         nb_cpu=1, clust_threshold=1, mmseqs_options=None,
                         linclust=None, propagate_annotation=None, uniref_annotation=None,
-                        expression_annotation=None, beta=None):
+                        expression_annotation=None, beta=None, minimal_number_proteomes=1):
     """From the proteomes found by esmecata proteomes, create protein cluster for each taxonomic affiliations.
 
     Args:
@@ -91,6 +105,7 @@ def perform_workflow(input_file, output_folder, busco_percentage_keep=80, ignore
         uniref_annotation (bool): option to use uniref annotation to add annotation
         expression_annotation (bool): option to add expression annotation from uniprot
         beta (bool): option to use the new API of UniProt (in beta can be unstable)
+        minimal_number_proteomes (int): minimal number of proteomes required to be associated with a taxon for the taoxn to be kepp
     """
     starttime = time.time()
     workflow_metadata = {}
@@ -101,7 +116,8 @@ def perform_workflow(input_file, output_folder, busco_percentage_keep=80, ignore
     proteomes_output_folder = os.path.join(output_folder, '0_proteomes')
     retrieve_proteomes(input_file, proteomes_output_folder, busco_percentage_keep,
                         ignore_taxadb_update, all_proteomes, uniprot_sparql_endpoint,
-                        remove_tmp, limit_maximal_number_proteomes, beta, rank_limit)
+                        remove_tmp, limit_maximal_number_proteomes, beta, rank_limit,
+                        minimal_number_proteomes)
 
     clustering_output_folder = os.path.join(output_folder, '1_clustering')
     make_clustering(proteomes_output_folder, clustering_output_folder, nb_cpu, clust_threshold, mmseqs_options, linclust, remove_tmp)
